#!/usr/bin/env python

import struct

class Offset:
    def __init__(self, typ, val):
        self.typ = typ
        self.val = val

class Vertex:
    def __init__(self, flags, x, y, z):
        self.flags = flags
        self.p = (x, y, z)
    def __repr__(self):
        return "%d %f %f %f" % (self.flags, self.p[0], self.p[1], self.p[2])
    def __str__(self):
        return "%d %f %f %f" % (self.flags, self.p[0], self.p[1], self.p[2])
    def __cmp__(self, v):
        return cmp(self.p, v.p)

def get_vertex(data):
    p = struct.unpack("<Hfff", data)
    return Vertex(*p)

class Triangle:
    def __init__(self, flags, v1, v2, v3):
        self.flags = flags
        self.v = (v1, v2, v3)
    def __str__(self):
        return "%d %d %d %d" % (self.flags, self.v[0], self.v[1], self.v[2])

def get_triangle(data):
    p = struct.unpack("<H3I", data)
    return Triangle(*p)

def get_data_fixed(f, offset, translate_fn):
    f.seek(offset)
    hdr = f.read(10)
    hdr = struct.unpack("<HII", hdr)
    num_elements = hdr[1]
    sz = hdr[2]
    print "data hdr", hdr
    els = []
    for i in range(num_elements):
        dc = f.read(sz)
        data = translate_fn(dc)
        els.append(data)
        print data
    return els

def open_mm3d(fname):
    f = open(fname)
    hdr = f.read(12)
    p = struct.unpack("<8cBBBB", hdr)
    print p
    offs = []
    for i in range(p[-1]): # data offsets
        o = f.read(6)
        print len(o), repr(o)
        o = struct.unpack("<HI", o)
        os = Offset(*o)
        print hex(o[0]), o[1]
        offs.append(os)

    for of in offs:
        if of.typ == 0x8001: # vertices
            vertices = get_data_fixed(f, of.val, get_vertex)
        if of.typ == 0x8021: # triangles
            triangles = get_data_fixed(f, of.val, get_triangle)

    return vertices, triangles

if __name__ == '__main__':
    open_mm3d("test.mm3d")
